package com.gic.cloud.data.hook.service.dao;

import com.gic.cloud.data.hook.api.dto.DownloadProcessDTO;
import com.gic.cloud.data.hook.api.dto.DownloadTask;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.Date;
import java.util.List;

/** 下载任务 DAO
 * @author Sam
 */
@Mapper
public interface DownloadTaskDao {

	/** 创建下载任务
	 * @param task
	 * @return
	 */
	public Integer createDownloadTask(DownloadTask task);

	/** 更新下载任务
	 * @param task
	 * @return
	 */
	public Integer updateDownloadTask(DownloadTask task);

	/** 删除下载任务
	 * @param taskId
	 * @return
	 */
	public Integer deleteDownloadTask(@Param("taskId") String taskId);

	/** 获取指定编号的下载任务
	 * @param taskId
	 * @return
	 */
	public DownloadTask getDownloadTaskById(@Param("taskId") String taskId);

	/** 获取指定用户的所有下载任务
	 * @param userId
	 * @param fuzzy
	 * @return
	 */
	public List<DownloadTask> getDownloadTaskListByUserId(@Param("userId")String userId, @Param("fuzzy") String fuzzy);

	/** 获取过期的下载任务列表
	 * @param expireDate
	 * @return
	 */
	public List<DownloadTask> getExpiredDownloadTaskList(@Param("expireDate") Date expireDate);

	/** 获取指定企业编号的所有下载任务统计
	 * @param enterpriseId
	 * @return
	 */
	public Integer countDownloadTaskByEnterpriseId(@Param("enterpriseId") String enterpriseId);

	/** 获取等待申请通过状态的任务
	 * @return
	 */
	public List<DownloadTask> getDownloadTaskOfWaiting(@Param("queryDataSource") String queryDataSource);

	DownloadProcessDTO getDownloadProcess();

	int updateTaskStatusError(@Param("idList") List<String> idList);

}
