package com.gic.cloud.data.hook.web;

import com.gic.clerk.api.constant.ThirdProjectEnum;
import com.gic.clerk.api.dto.AuthorizedUser;
import com.gic.clerk.api.dto.PowerClerkDTO;
import com.gic.clerk.api.dto.right.RightMenuDTO;
import com.gic.clerk.api.service.PowerService;
import com.gic.clerk.api.service.RightService;
import com.gic.cloud.common.api.base.Page;
import com.gic.cloud.data.hook.api.dto.*;
import com.gic.cloud.data.hook.api.entity.FlatQueryCondition;
import com.gic.cloud.data.hook.api.entity.FlatQueryExecuteRequest;
import com.gic.cloud.data.hook.api.entity.GeneralResult;
import com.gic.cloud.data.hook.api.entity.Global;
import com.gic.cloud.data.hook.api.service.IFlatQueryResultService;
import com.gic.cloud.data.hook.api.service.IFlatQueryTableService;
import com.gic.cloud.data.hook.api.service.IMyFlatQueryService;
import com.gic.cloud.data.hook.api.service.SearchLogService;
import com.gic.enterprise.api.constant.StoreWidget;
import com.gic.enterprise.api.dto.EnterpriseSettingDTO;
import com.gic.enterprise.api.dto.StoreWidgetDTO;
import com.gic.enterprise.api.service.EnterpriseService;
import com.gic.enterprise.api.service.StoreWidgetService;
import com.gic.web.common.utils.SessionContextUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

/** 自助取数相关 Controller
 * @author Sam
 */
@Controller
public class FlatQueryController {

	/** 脱敏字段 */
	public static final List<String> FILTERS_PHONE_ONLY = Arrays.asList("mobile", "phone", "enterprise_name", "phone_number", "receive_phone_number", "use_phone_number", "mbr_phone");

	/** 脱敏字段 */
	public static final List<String> FILTERS_PHONE_AND_CARD = Arrays.asList("card_num", "mobile", "phone", "enterprise_name", "phone_number", "receive_phone_number","receive_card_num","use_phone_number", "use_card_num", "mbr_card_no");

	/** 自助查询结果与字段过滤服务 */
	@SuppressWarnings("SpringJavaInjectionPointsAutowiringInspection")
	@Autowired
	private IFlatQueryResultService flatQueryResultService;

	/** 自助查询表与字段定义服务 */
	@SuppressWarnings("SpringJavaInjectionPointsAutowiringInspection")
	@Autowired
	private IFlatQueryTableService flatQueryTableService;

	/** 自定义指标查询服务 */
	@SuppressWarnings("SpringJavaInjectionPointsAutowiringInspection")
	@Autowired
	private IMyFlatQueryService myFlatQueryService;
	@Autowired
	private EnterpriseService enterpriseService;
	@Autowired
	private StoreWidgetService storeWidgetService;
	@Autowired
	private PowerService powerService;
	@Autowired
	private RightService rightService;
	@Autowired
	private SearchLogService searchLogService;


	/** 查询表定义及下载量等信息
	 * @param userId 指定的用户主键
	 * @param fuzzy 模糊查询信息
	 * @param pageNum 指定分页数量
	 * @param pageSize 指定每页查询数量
	 */
	@RequestMapping("/flat-query-table-page")
	public Page<FlatQueryTable> flatQueryTablePage(String userId, String fuzzy, Integer pageNum, Integer pageSize, HttpServletRequest request, HttpServletResponse response) {
		List<RightMenuDTO> list = this.rightService.listSuperAdminAllGicWebMenu(SessionContextUtils.getLoginUserEnterpriseId(), ThirdProjectEnum.DATA_HOOK.getCode());
		List<String> tables = new ArrayList<>();
		if(org.apache.commons.collections.CollectionUtils.isNotEmpty(list)){
			for(RightMenuDTO menuDTO : list){
				if(SessionContextUtils.getLoginUser().getSuperAdmin() != 1){
					if("extract_integral_cost_bill_m".equals(menuDTO.getMenuCode())){
						continue;
					}
				}
				tables.add(menuDTO.getMenuCode());
			}
		}
		if(CollectionUtils.isEmpty(tables)){
			tables.add("-1");
		}
		return this.flatQueryTableService.queryFlatQueryTablePage(userId, fuzzy, pageNum, pageSize, tables); // 限定 10 个/分页
	}

	/** 切换自助查询表的收藏状态
	 * @param userId 指定的用户主键
	 * @param tableId 指定的表编号
	 * @param request
	 * @param response
	 * @return
	 */
	@RequestMapping("/flat-query-switch-favo")
	public GeneralResult flatQuerySwitchFavo(String userId, String tableId, HttpServletRequest request, HttpServletResponse response) {
		GeneralResult result = new GeneralResult();
		result.setResult(true); // 一定返回处理成功
		FlatQueryFavo favo = this.flatQueryTableService.queryFlatQueryFavo(userId, tableId);
		if (favo != null) { // 如果原来有值
			this.flatQueryTableService.removeFlatQueryFavo(userId, tableId);
			result.setInfo("0|" + tableId); // 0 表示取消收藏
		} else {
			this.flatQueryTableService.createFlatQueryFavo(userId, tableId);
			result.setInfo("1|" + tableId); // 1 表示添加收藏
		} // IF ELSE OVER
		return result;
	}

	/** 获取指定自助查询表明细
	 * @param userId 用户主键
	 * @param tableId 指定表名
	 * @param request
	 * @param response
	 * @return
	 */
	@RequestMapping("/flat-query-table-detail")
	public FlatQueryTableDetail flatQueryTableDetail(String userId, String tableId, HttpServletRequest request, HttpServletResponse response) {
		System.out.println(userId);
		System.out.println(tableId);
		return this.flatQueryTableService.queryFlatQueryTableDetailByTableId(userId, tableId);
	}

	/** 执行自助查询
	 * @param executeRequest
	 * @param request
	 * @param response
	 * @return
	 */
	@RequestMapping("/flat-query")
	public FlatQueryResult doFlatQuery(@RequestBody FlatQueryExecuteRequest executeRequest, HttpServletRequest request, HttpServletResponse response) {
		System.out.println("doFlatQuery:" + executeRequest.getTableId());
		System.out.println("doFlatQuery:" + executeRequest.getQueryConditions().size());
		String enterpriseId = SessionContextUtils.getLoginUserEnterpriseId();
		EnterpriseSettingDTO setting = this.enterpriseService.getEnterpriseSettingByEnterpriseId(enterpriseId);
		int dataPermission = 0;
		if(setting.getDataPermission() != null){
			dataPermission = setting.getDataPermission();
		}
		for (FlatQueryCondition condition : executeRequest.getQueryConditions()) {
			System.out.println(condition.getFieldName() + " " + condition.getFieldType());
		}
		AuthorizedUser loginUser = SessionContextUtils.getLoginUser();
		DhSearchLogDTO dto = new DhSearchLogDTO();
		dto.setEnterpriseId(loginUser.getEnterpriseId());
		dto.setEnterpriseName(loginUser.getEnterpriseDTO().getEnterpriseName());
		dto.setUserId(loginUser.getUserId());
		dto.setUserName(loginUser.getLoginName());
		dto.setModel(executeRequest.getModel());
		dto.setCount(executeRequest.getCount());
		dto.setStartTime(new Date());
		FlatQueryResult flatQueryResult = this.flatQueryResultService.getFlatQueryResult(
				executeRequest.getTableId(),
				executeRequest.getEnterpriseIds(),
				executeRequest.getQueryConditions(),
				executeRequest.getOrderField(),
				executeRequest.getOrderDir(),
				executeRequest.getExecDistinct(),
				executeRequest.getQueryDataType(),
				1000,
				dataPermission,
				this.getAuthStoreId());// 强制 1000 限制
		dto.setEndTime(new Date());
		this.searchLogService.saveLog(dto);
		return flatQueryResult;
	}

	/** 获取自助查询下载量评估
	 * @param executeRequest
	 * @param request
	 * @param response
	 * @return
	 */
	@RequestMapping("/get-flat-query-download-task-amount")
	public GeneralResult getFlatQueryDownloadTaskAmount(@RequestBody FlatQueryExecuteRequest executeRequest, HttpServletRequest request, HttpServletResponse response) {
		GeneralResult result = new GeneralResult();
		Integer amount = this.flatQueryResultService.estimateFlatQueryDownloadTask(
				executeRequest.getTableId(),
				executeRequest.getEnterpriseIds(),
				executeRequest.getQueryConditions(),
				executeRequest.getOrderField(),
				executeRequest.getOrderDir(),
				executeRequest.getExecDistinct(),
				this.getAuthStoreId()
			);
		result.setInfo(String.valueOf(amount));
		return result;
	}

	/** 生成自助查询下载任务
	 * @param executeRequest
	 * @param request
	 * @param response
	 * @return
	 */
	@RequestMapping("/build-flat-query-download-task")
	public DownloadTask buildDownloadTask(@RequestBody FlatQueryExecuteRequest executeRequest, HttpServletRequest request, HttpServletResponse response) {
		List<FlatQueryCondition> queryConditions = executeRequest.getQueryConditions();
		Integer dataPermission = this.enterpriseService.getEnterpriseSettingByEnterpriseId(SessionContextUtils.getLoginUserEnterpriseId()).getDataPermission();
		if(dataPermission != null){
			for(FlatQueryCondition condition : queryConditions){
				if(dataPermission == 1){
					if(FILTERS_PHONE_ONLY.contains(condition.getFieldName())){
						condition.setEnableEncrypt(true);
					}else {
						condition.setEnableEncrypt(false);
					}
				}
				if(dataPermission == 2){
					if(FILTERS_PHONE_AND_CARD.contains(condition.getFieldName())){
						condition.setEnableEncrypt(true);
					} else {
						condition.setEnableEncrypt(false);
					}
				}
			}
		}
		return this.flatQueryResultService.buildFlatQueryDownloadTask(
				executeRequest.getUserId(),
				executeRequest.getName(),
				executeRequest.getModuleName(),
				executeRequest.getAmount(),
				executeRequest.getFormat(),
				executeRequest.getTableId(),
				executeRequest.getEnterpriseIds(),
				queryConditions,
				executeRequest.getOrderField(),
				executeRequest.getOrderDir(),
				executeRequest.getExecDistinct(),
				executeRequest.getUseCompress() ? Global.YES : Global.NO,
				executeRequest.getQueryDataType(),
				executeRequest.getApplyId(),
				this.getAuthStoreId()
		);
	}


	/** 获取自助查询字段可过滤值
	 * @param tableId
	 * @param fieldName
	 * @param request
	 * @param response
	 * @return
	 */
	@RequestMapping("/flat-query-filter-data")
	public List<Object> getFlatQueryTextFilterData(@RequestParam String tableId, @RequestParam String enterpriseIdsText, @RequestParam String fieldName, HttpServletRequest request, HttpServletResponse response) {
		String[] enterpriseIdsArr = enterpriseIdsText.split(",");
		return this.flatQueryResultService.getFlatQueryFilterData(tableId, CollectionUtils.arrayToList(enterpriseIdsArr), fieldName);
	}

	/** 查询自定义指标查询
	 * @param id 指定的自定义指标
	 */
	@RequestMapping("/my-flat-query")
	public MyFlatQuery myFlatQuery(@RequestParam String id, HttpServletRequest request, HttpServletResponse response) {
		return this.myFlatQueryService.queryMyFlatQueryById(id);
	}

	/** 查询自定义指标查询分页
	 * @param userId 指定的用户主键
	 * @param fuzzy 模糊查询信息
	 * @param pageNum 指定分页数量
	 * @param pageSize 指定每页查询数量
	 */
	@RequestMapping("/my-flat-query-page")
	public Page<MyFlatQuery> myFlatQueryPage(@RequestParam String userId, @RequestParam String fuzzy, @RequestParam Integer pageNum, @RequestParam Integer pageSize, HttpServletRequest request, HttpServletResponse response) {
		return this.myFlatQueryService.queryMyFlatQueryPage(userId, fuzzy, pageNum, pageSize); // 限定 10 个/分页
	}

	/** 创建或保存自定义指标查询
	 * @param myFlatQuery
	 * @param request
	 * @param response
	 * @return
	 */
	@RequestMapping("/save-my-flat-query")
	public GeneralResult saveMyFlatQuery(@RequestBody MyFlatQuery myFlatQuery, HttpServletRequest request, HttpServletResponse response) {
		GeneralResult result = new GeneralResult();
		Integer count = 0;
		if (myFlatQuery.getId().equals("")) { // 如果是新建
			myFlatQuery.setId(String.valueOf((new Date()).getTime()));
			myFlatQuery.setUpdateTime(new Date());
			count = myFlatQueryService.createMyFlatQuery(myFlatQuery);
		} else { // 如果是更新
			myFlatQuery.setUpdateTime(new Date());
			count = myFlatQueryService.updateMyFlatQuery(myFlatQuery);
		} // IF ELSE OVER
		result.setResult(true);
		result.setInfo(myFlatQuery.getId());
		return result;
	}

	/** 删除指定的自定义指标查询
	 * @param id
	 * @param request
	 * @param response
	 * @return
	 */
	@RequestMapping("/delete-my-flat-query")
	public GeneralResult deleteMyFlatQuery(@RequestParam String id, HttpServletRequest request, HttpServletResponse response) {
		GeneralResult result = new GeneralResult();
		Integer isSuccess = myFlatQueryService.deleteMyFlatQuery(id);
		result.setInfo(isSuccess.toString());
		return result;
	}


	private List<String> getAuthStoreId(){
		List<String> list = new ArrayList<>();
		AuthorizedUser user = SessionContextUtils.getLoginUser();
		if(user.getSuperAdmin() != 1){
			EnterpriseSettingDTO setting = this.enterpriseService.getEnterpriseSettingByEnterpriseId(user.getEnterpriseId());
			if(setting.getEnableAccessControl() != null && setting.getEnableAccessControl() == 1) {
				PowerClerkDTO clerkDetail = this.powerService.getClerkDetail(user.getUserId());
				StoreWidgetDTO widgetDTO = this.storeWidgetService.getStoreWidgetBykey(clerkDetail.getStoreWidgetId());
				if(StoreWidget.ALL != widgetDTO.getSelectType()){
					List<String> authStoreList = this.storeWidgetService.getStoreWidgetStore(clerkDetail.getStoreWidgetId(), user.getEnterpriseId(), null, null, null);
					list.addAll(authStoreList);
					if(CollectionUtils.isEmpty(list)){
						list.add("1");
					}
				}
			}
		}
		return list;
	}



}
